import re

from github.Label import Label
from github.PullRequest import PullRequest


def lint(pr: PullRequest) -> None:
    """
    Lint a pull request:
    1. checks if the PR title ends with a username (owner)
    2. The username must be a collaborator
    3. checks if there is exactly one Type label
    4. checks if there is exactly one Impact label

    Args:
        pr: The pull request to lint
    """

    title = pr.title
    repo = pr.base.repo

    pr_owners = re.findall(r"(@[\w]+)$", title)
    assert pr_owners, "PR title should end with a GitHub username"
    pr_owner = pr_owners[0][1:]
    assert repo.has_in_collaborators(pr_owner), f"{pr_owner} is not a collaborator"

    labels = pr.get_labels()

    type_labels: list[Label] = []
    impact_labels: list[Label] = []

    for label in labels:
        if "Type:" in label.name:
            type_labels.append(label)
        elif "Impact:" in label.name:
            impact_labels.append(label)

    assert len(type_labels) == 1, "There should be exactly one Type label"
    assert len(impact_labels) == 1, "There should be exactly one Impact label"

    # TODO: check if the PR title is in the correct format (e.g. commitlint convention)



import re

from github.PullRequest import PullRequest

emoji_pattern = re.compile(
    "["
    "\U0001F600-\U0001F64F"  # Emoticons
    "\U0001F300-\U0001F5FF"  # Symbols & pictographs
    "\U0001F680-\U0001F6FF"  # Transport & map symbols
    "\U0001F1E0-\U0001F1FF"  # Flags (iOS)
    "\U0001F900-\U0001F9FF"  # Supplemental Symbols and Pictographs
    "\U0001FA70-\U0001FAFF"  # Symbols and Pictographs Extended-A
    "\u2600-\u26FF"  # Miscellaneous Symbols
    "\u2700-\u27BF"  # Dingbats
    "]"
)


def format(pr: PullRequest) -> None:
    """
    Format a pull request:

    1. Extract emojis from labels and insert them to the PR title

    Args:
        pr: The pull request to format
    """

    labels = pr.get_labels()
    # remove all emojis from the left of title
    cleaned_title = pr.title.lstrip("".join(emoji_pattern.findall(pr.title))).strip()

    emojis = set()
    for label in labels:
        if _emjojis := emoji_pattern.findall(label.name):
            emojis.update(_emjojis)

    new_title = f"{''.join(emojis)} {cleaned_title}"
    if new_title != pr.title:
        pr.edit(title=new_title)
        print(f"Updated PR title: {pr.title} -> {new_title}")



import os

import typer
from github import Github

from .formatter import format
from .linter import lint

app = typer.Typer()


@app.command()
def main() -> None:
    token = os.getenv("GITHUB_TOKEN")
    repo_name = os.getenv("GITHUB_REPOSITORY")
    pr_number = os.getenv("PR_NUMBER")

    assert token, "Missing GITHUB_TOKEN environment variable"
    assert repo_name, "Missing GITHUB_REPOSITORY environment variable"
    assert pr_number, "Missing PR_NUMBER environment variable"

    # Authenticate to GitHub
    g = Github(token)

    # Get the repository
    repo = g.get_repo(repo_name)

    # Get the pull request
    pr = repo.get_pull(int(pr_number))

    lint(pr)
    format(pr)


if __name__ == "__main__":
    app()
